/*
 * pppd.h - PPP daemon global declarations.
 *
 * Copyright (c) 1989 Carnegie Mellon University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by Carnegie Mellon University.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * $Id$
 */

#ifndef __PPPD_H__
#define __PPPD_H__

#include <stdbool.h>		/* bool */
#include <stdio.h>		/* for FILE */
#include <limits.h>		/* for NGROUPS_MAX */
#include <sys/param.h>		/* for MAXPATHLEN and BSD4_4, if defined */
#include <sys/types.h>		/* for uint32_t, if defined */
#include <sys/time.h>		/* for struct timeval */
#include <net/ppp_defs.h>
#include <rtems/rtemsdialer.h>
#include <stdint.h>

#if defined(__STDC__)
#include <stdarg.h>
#define __V(x)	x
#else
#include <varargs.h>
#define __V(x)	(va_alist) va_dcl
#define const
#define volatile
#endif

#ifdef INET6
#include "eui64.h"
#endif

/*
 * Limits.
 */

#define NUM_PPP		1	/* One PPP interface supported (per process) */
#define MAXWORDLEN	1024	/* max length of word in file (incl null) */
#define MAXARGS		1	/* max # args to a command */
#define MAXNAMELEN	256	/* max length of hostname or name for auth */
#define MAXSECRETLEN	256	/* max length of password or secret */

/*
 * Option descriptor structure.
 */

enum opt_type {
	o_special_noarg = 0,
	o_special = 1,
	o_bool,
	o_int,
	o_uint32,
	o_string,
};

typedef struct {
	char	*name;		/* name of the option */
	enum opt_type type;
	void	*addr;
	char	*description;
	uint32_t flags;
	void	*addr2;
	int	upper_limit;
	int	lower_limit;
} option_t;

/* Values for flags */
#define OPT_VALUE	0xffL	/* mask for presupplied value */
#define OPT_HEX		0x100L	/* int option is in hex */
#define OPT_NOARG	0x200L	/* option doesn't take argument */
#define OPT_OR		0x400L	/* OR in argument to value */
#define OPT_INC		0x800L	/* increment value */
#define OPT_PRIV	0x1000L	/* privileged option */
#define OPT_STATIC	0x2000L	/* string option goes into static array */
#define OPT_LLIMIT	0x4000L	/* check value against lower limit */
#define OPT_ULIMIT	0x8000L	/* check value against upper limit */
#define OPT_LIMITS	(OPT_LLIMIT|OPT_ULIMIT)
#define OPT_ZEROOK	0x10000L	/* 0 value is OK even if not within limits */
#define OPT_NOINCR	0x20000L	/* value mustn't be increased */
#define OPT_ZEROINF	0x40000L	/* with OPT_NOINCR, 0 == infinity */
#define OPT_A2INFO	0x100000L /* addr2 -> option_info to update */
#define OPT_A2COPY	0x200000L /* addr2 -> second location to rcv value */
#define OPT_ENABLE	0x400000L /* use *addr2 as enable for option */
#define OPT_PRIVFIX	0x800000L /* can't be overridden if noauth */
#define OPT_PREPASS	0x1000000L /* do this opt in pre-pass to find device */
#define OPT_INITONLY	0x2000000L /* option can only be set in init phase */
#define OPT_DEVEQUIV	0x4000000L /* equiv to device name */
#define OPT_DEVNAM	(OPT_PREPASS | OPT_INITONLY | OPT_DEVEQUIV)

#define OPT_VAL(x)	((x) & OPT_VALUE)

#ifndef GIDSET_TYPE
#define GIDSET_TYPE	gid_t
#endif

/* Structure representing a list of permitted IP addresses. */
struct permitted_ip {
    int		permit;		/* 1 = permit, 0 = forbid */
    uint32_t	base;		/* match if (addr & mask) == base */
    uint32_t	mask;		/* base and mask are in network byte order */
};

/*
 * Unfortunately, the linux kernel driver uses a different structure
 * for statistics from the rest of the ports.
 * This structure serves as a common representation for the bits
 * pppd needs.
 */
struct pppd_stats {
    unsigned int	bytes_in;
    unsigned int	bytes_out;
};

/* Used for storing a sequence of words.  Usually malloced. */
struct wordlist {
    struct wordlist	*next;
    char		*word;
};

/*
 * Global variables.
 */

extern int	pppd_kill_link;	/* Signal to terminate processing loop */
extern int	hungup;		/* Physical layer has disconnected */
extern int	pppifunit;	/* Interface unit number */
extern char	ifname[];	/* Interface name */
extern int	pppd_ttyfd;	/* Serial device file descriptor */
extern char	hostname[];	/* Our hostname */
extern u_char	outpacket_buf[]; /* Buffer for outgoing packets */
extern int	pppd_phase;		/* Current state of link - see values below */
extern int	baud_rate;	/* Current link speed in bits/sec */
extern int	redirect_stderr;/* Connector's stderr should go to file */
extern char	peer_authname[];/* Authenticated name of peer */
extern int	privileged;	/* We were run by real-uid root */
extern int	need_holdoff;	/* Need holdoff period after link terminates */
extern char	**script_env;	/* Environment variables for scripts */
extern int	detached;	/* Have detached from controlling tty */
extern GIDSET_TYPE groups[NGROUPS_MAX];	/* groups the user is in */
extern int	ngroups;	/* How many groups valid in groups */
extern struct pppd_stats link_stats; /* byte/packet counts etc. for link */
extern int	using_pty;	/* using pty as device (notty or pty opt.) */
extern int	log_to_fd;	/* logging to this fd as well as syslog */
extern char	*no_ppp_msg;	/* message to print if ppp not in kernel */
extern volatile int pppd_status;	/* exit status for pppd */
extern int	devnam_fixed;	/* can no longer change devnam */
extern int	unsuccess;	/* # unsuccessful connection attempts */
extern int	do_callback;	/* set if we want to do callback next */
extern int	doing_callback;	/* set if this is a callback */
extern dialerfp pppd_dialer;    /* script dialer function callback */

/* Values for do_callback and doing_callback */
#define CALLBACK_DIALIN		1	/* we are expecting the call back */
#define CALLBACK_DIALOUT	2	/* we are dialling out to call back */

/*
 * Variables set by command-line options.
 */

extern int	debug;		/* Debug flag */
extern int	kdebugflag;	/* Tell kernel to print debug messages */
extern int	default_device;	/* Using /dev/tty or equivalent */
extern char	devnam[MAXPATHLEN];	/* Device name */
extern int	crtscts;	/* Use hardware flow control */
extern bool	modem;		/* Use modem control lines */
extern int	inspeed;	/* Input/Output speed requested */
extern uint32_t netmask;	/* IP netmask to set on interface */
extern bool	lockflag;	/* Create lock file to lock the serial dev */
extern bool	nodetach;	/* Don't detach from controlling tty */
extern bool	updetach;	/* Detach from controlling tty when link up */
extern char	*initializer;	/* Script to initialize physical link */
extern char	*connect_script; /* Script to establish physical link */
extern char	*disconnect_script; /* Script to disestablish physical link */
extern char	*welcomer;	/* Script to welcome client after connection */
extern char	*ptycommand;	/* Command to run on other side of pty */
extern int	maxconnect;	/* Maximum connect time (seconds) */
extern char	user[MAXNAMELEN];/* Our name for authenticating ourselves */
extern char	passwd[MAXSECRETLEN];	/* Password for PAP or CHAP */
extern bool	auth_required;	/* Peer is required to authenticate */
extern bool	persist;	/* Reopen link after it goes down */
extern bool	uselogin;	/* Use /etc/passwd for checking PAP */
extern char	our_name[MAXNAMELEN];/* Our name for authentication purposes */
extern char	remote_name[MAXNAMELEN]; /* Peer's name for authentication */
extern bool	explicit_remote;/* remote_name specified with remotename opt */
extern bool	demand;		/* Do dial-on-demand */
extern char	*ipparam;	/* Extra parameter for ip up/down scripts */
extern bool	cryptpap;	/* Others' PAP passwords are encrypted */
extern int	idle_time_limit;/* Shut down link if idle for this long */
extern int	holdoff;	/* Dead time before restarting */
extern bool	holdoff_specified; /* true if user gave a holdoff value */
extern bool	notty;		/* Stdin/out is not a tty */
extern char	*record_file;	/* File to record chars sent/received */
extern bool	sync_serial;	/* Device is synchronous serial device */
extern int	maxfail;	/* Max # of unsuccessful connection attempts */
extern char	linkname[MAXPATHLEN]; /* logical name for link */
extern bool	tune_kernel;	/* May alter kernel settings as necessary */
extern int	connect_delay;	/* Time to delay after connect script */

#ifdef PPP_FILTER
extern struct	bpf_program pass_filter;   /* Filter for pkts to pass */
extern struct	bpf_program active_filter; /* Filter for link-active pkts */
#endif

#ifdef MSLANMAN
extern bool	ms_lanman;	/* Use LanMan password instead of NT */
				/* Has meaning only with MS-CHAP challenges */
#endif

extern char *current_option;	/* the name of the option being parsed */
extern int  privileged_option;	/* set iff the current option came from root */
extern char *option_source;	/* string saying where the option came from */

/*
 * Values for phase.
 */
#define PHASE_DEAD		0
#define PHASE_INITIALIZE	1
#define PHASE_SERIALCONN	2
#define PHASE_DORMANT		3
#define PHASE_ESTABLISH		4
#define PHASE_AUTHENTICATE	5
#define PHASE_CALLBACK		6
#define PHASE_NETWORK		7
#define PHASE_RUNNING		8
#define PHASE_TERMINATE		9
#define PHASE_DISCONNECT	10
#define PHASE_HOLDOFF		11

/*
 * The following struct gives the addresses of procedures to call
 * for a particular protocol.
 */
struct protent {
    u_short protocol;		/* PPP protocol number */
    /* Initialization procedure */
    void (*init)(int unit);
    /* Process a received packet */
    void (*input)(int unit, u_char *pkt, int len);
    /* Process a received protocol-reject */
    void (*protrej)(int unit);
    /* Lower layer has come up */
    void (*lowerup)(int unit);
    /* Lower layer has gone down */
    void (*lowerdown)(int unit);
    /* Open the protocol */
    void (*open)(int unit);
    /* Close the protocol */
    void (*close)(int unit, char *reason);
    /* Print a packet in readable form */
    int  (*printpkt)(u_char *pkt, int len,
			  void (*printer)(void *, char *, ...),
			  void *arg);
    /* Process a received data packet */
    void (*datainput)(int unit, u_char *pkt, int len);
    bool enabled_flag;		/* 0 iff protocol is disabled */
    char *name;			/* Text name of protocol */
    char *data_name;		/* Text name of corresponding data protocol */
    option_t *options;		/* List of command-line options */
    /* Check requested options, assign defaults */
    void (*check_options)(void);
    /* Configure interface for demand-dial */
    int  (*demand_conf)(int unit);
    /* Say whether to bring up link for this pkt */
    int  (*active_pkt)(u_char *pkt, int len);
};

/* Table of pointers to supported protocols */
extern struct protent *protocols[];

/*
 * Prototypes.
 */

/* Procedures exported from main.c. */
void die(int);		/* Cleanup and exit */
void quit(void);		/* like die(1) */
void novm(char *);	/* Say we ran out of memory, and die */
void ppptimeout(void (*func)(void *), void *arg, int t);
				/* Call func(arg) after t seconds */
void pppuntimeout(void (*func)(void *), void *arg);
				/* Cancel call to func(arg) */
void update_link_stats(int); /* Get stats at link termination */
void new_phase(int);	/* signal start of new phase */

/* Procedures exported from utils.c. */
void log_packet(u_char *, int, char *, int);
				/* Format a packet and log it with syslog */
void print_string(void *, int,  void (*) (void *, char *, ...),
		void *);	/* Format a string for output */
int slprintf(char *, int, char *, ...);		/* sprintf++ */
int vslprintf(char *, int, char *, va_list);	/* vsprintf++ */
size_t strlcpy(char *, const char *, size_t);	/* safe strcpy */
size_t strlcat(char *, const char *, size_t);	/* safe strncpy */
void pppd_dbglog(char *, ...);	/* log a debug message */
void pppd_info(char *, ...);	/* log an informational message */
void pppd_notice(char *, ...);	/* log a notice-level message */
void pppd_warn(char *, ...);	/* log a warning message */
void pppd_error(char *, ...);	/* log an error message */
void pppd_fatal(char *, ...);	/* log an error message and die(1) */

#define dbglog pppd_dbglog
#define info   pppd_info
#define notice pppd_notice
#define warn   pppd_warn
#define error  pppd_error
#define fatal  pppd_fatal

/* Procedures exported from auth.c */
void link_required(int);	  /* we are starting to use the link */
void link_terminated(int);  /* we are finished with the link */
void link_down(int);	  /* the LCP layer has left the Opened state */
void link_established(int); /* the link is up; authenticate now */
void start_networks(void);  /* start all the network control protos */
void np_up(int, int);	  /* a network protocol has come up */
void np_down(int, int);	  /* a network protocol has gone down */
void np_finished(int, int); /* a network protocol no longer needs link */
void auth_peer_fail(int, int);
				/* peer failed to authenticate itself */
void auth_peer_success(int, int, char *, int);
				/* peer successfully authenticated itself */
void auth_withpeer_fail(int, int);
				/* we failed to authenticate ourselves */
void auth_withpeer_success(int, int);
				/* we successfully authenticated ourselves */
int  auth_check_options(void);
				/* check authentication options supplied */
void auth_reset(int);	/* check what secrets we have */
int  check_passwd(int, char *, int, char *, int, char **);
				/* Check peer-supplied username/password */
int  get_secret(int, char *, char *, unsigned char *, int *, int);
				/* get "secret" for chap */
int  auth_ip_addr(int, uint32_t);
				/* check if IP address is authorized */
int  bad_ip_adrs(uint32_t);
				/* check if IP address is unreasonable */

/* Procedures exported from demand.c */
void demand_conf(void);	/* config interface(s) for demand-dial */
void demand_block(void);	/* set all NPs to queue up packets */
void demand_unblock(void); /* set all NPs to pass packets */
void demand_discard(void); /* set all NPs to discard packets */
void demand_rexmit(int);	/* retransmit saved frames for an NP */
int  loop_chars(unsigned char *, int); /* process chars from loopback */
int  loop_frame(unsigned char *, int); /* should we bring link up? */

/* Procedures exported from sys-*.c */
void sys_init(void);	/* Do system-dependent initialization */
void sys_cleanup(void);	/* Restore system state before exiting */
int  sys_check_options(void); /* Check options specified */
void sys_close(void);	/* Clean up in a child before execing */
int  ppp_available(void);	/* Test whether ppp kernel support exists */
int  get_pty(int *, int *, char *, int);	/* Get pty master/slave */
int  open_ppp_loopback(void); /* Open loopback for demand-dialling */
int  establish_ppp(int);	/* Turn serial port into a ppp interface */
void restore_loop(void);	/* Transfer ppp unit back to loopback */
void disestablish_ppp(int); /* Restore port to normal operation */
void clean_check(void);	/* Check if line was 8-bit clean */
void set_up_tty(int, int); /* Set up port's speed, parameters, etc. */
void restore_tty(int);	/* Restore port's original parameters */
void setdtr(int, int);	/* Raise or lower port's DTR line */
void output(int, u_char *, int); /* Output a PPP packet */
void wait_input(struct timeval *); /* Wait for input, with timeout */

void ppp_delay(void);       /* delay task for a little while */
int  read_packet(u_char *); /* Read PPP packet */
int  get_loop_output(void); /* Read pkts from loopback */
void ppp_send_config(int, int, uint32_t, int, int);
				/* Configure i/f transmit parameters */
void ppp_set_xaccm(int, ext_accm);
				/* Set extended transmit ACCM */
void ppp_recv_config(int, int, uint32_t, int, int);
				/* Configure i/f receive parameters */
int  ccp_test(int, u_char *, int, int);
				/* Test support for compression scheme */
void ccp_flags_set(int, int, int);
				/* Set kernel CCP state */
int  ccp_fatal_error(int); /* Test for fatal decomp error in kernel */
int  get_idle_time(int, struct ppp_idle *);
				/* Find out how long link has been idle */
int  get_ppp_stats(int, struct pppd_stats *);
				/* Return link statistics */
int  sifvjcomp(int, int, int, int);
				/* Configure VJ TCP header compression */
int  sifup(int);		/* Configure i/f up for one protocol */
int  sifnpmode(int u, int proto, enum NPmode mode);
				/* Set mode for handling packets for proto */
int  sifdown(int);	/* Configure i/f down for one protocol */
int  sifaddr(int, uint32_t, uint32_t, uint32_t);
				/* Configure IPv4 addresses for i/f */
int  cifaddr(int, uint32_t, uint32_t);
				/* Reset i/f IP addresses */
#ifdef INET6
int  sif6addr(int, eui64_t, eui64_t);
				/* Configure IPv6 addresses for i/f */
int  cif6addr(int, eui64_t, eui64_t);
				/* Remove an IPv6 address from i/f */
#endif
int  sifdefaultroute(int, uint32_t, uint32_t);
				/* Create default route through i/f */
int  cifdefaultroute(int, uint32_t, uint32_t);
				/* Delete default route through i/f */
int  sifproxyarp(int, uint32_t);
				/* Add proxy ARP entry for peer */
int  cifproxyarp(int, uint32_t);
				/* Delete proxy ARP entry for peer */
uint32_t GetMask(uint32_t); /* Get appropriate netmask for address */
int  lock(char *);	/* Create lock file for device */
int  relock(int);		/* Rewrite lock file with new pid */
void unlock(void);	/* Delete previously-created lock file */
void logwtmp(const char *, const char *, const char *);
				/* Write entry to wtmp file */
int  get_host_seed(void);	/* Get host-dependent random number seed */
int  have_route_to(uint32_t); /* Check if route to addr exists */
#ifdef PPP_FILTER
int  set_filters(struct bpf_program *pass, struct bpf_program *active);
				/* Set filter programs in kernel */
#endif
#ifdef IPX_CHANGE
int  sipxfaddr(int, unsigned long, unsigned char *);
int  cipxfaddr(int);
#endif

/* Procedures exported from options.c */
int  parse_args(int argc, char **argv);
				/* Parse options from arguments given */
int  options_from_file(char *filename, int must_exist, int check_prot,
			    int privileged);
				/* Parse options from an options file */
int  options_from_user(void); /* Parse options from user's .ppprc */
int  options_for_tty(void); /* Parse options from /etc/ppp/options.tty */
int  options_from_list(struct wordlist *, int privileged);
				/* Parse options from a wordlist */
int  getword(FILE *f, char *word, int *newlinep, char *filename);
				/* Read a word from a file */
void option_error(char *fmt, ...);
				/* Print an error message about an option */
int int_option(char *, int *);
				/* Simplified number_option for decimal ints */
void add_options(option_t *); /* Add extra options */

/*
 * This structure is used to store information about certain
 * options, such as where the option value came from (/etc/ppp/options,
 * command line, etc.) and whether it came from a privileged source.
 */

struct option_info {
    int	    priv;		/* was value set by sysadmin? */
    char    *source;		/* where option came from */
};

extern struct option_info devnam_info;
extern struct option_info initializer_info;
extern struct option_info connect_script_info;
extern struct option_info disconnect_script_info;
extern struct option_info welcomer_info;
extern struct option_info ptycommand_info;

/*
 * Hooks to enable plugins to change various things.
 */
extern int (*new_phase_hook)(int);
extern int (*idle_time_hook)(struct ppp_idle *);
extern int (*holdoff_hook)(void);
extern int (*pap_check_hook)(void);
extern int (*pap_auth_hook)(char *user, char *passwd/*, char **msgp,
				 struct wordlist **paddrs,
				 struct wordlist **popts*/);
extern void (*pap_logout_hook)(void);
extern int (*pap_passwd_hook)(char *user, char *passwd);
extern void (*ip_up_hook)(void);
extern void (*ip_down_hook)(void);
extern void (*auth_linkup_hook)(void);
extern void (*auth_linkdown_hook)(void);

/*
 * Inline versions of get/put char/short/long.
 * Pointer is advanced; we assume that both arguments
 * are lvalues and will already be in registers.
 * cp MUST be u_char *.
 */
#define GETCHAR(c, cp) { \
	(c) = *(cp)++; \
}
#define PUTCHAR(c, cp) { \
	*(cp)++ = (u_char) (c); \
}


#define GETSHORT(s, cp) { \
	(s) = *(cp)++ << 8; \
	(s) |= *(cp)++; \
}
#define PUTSHORT(s, cp) { \
	*(cp)++ = (u_char) ((s) >> 8); \
	*(cp)++ = (u_char) (s); \
}

#define GETLONG(l, cp) { \
	(l) = *(cp)++ << 8; \
	(l) |= *(cp)++; (l) <<= 8; \
	(l) |= *(cp)++; (l) <<= 8; \
	(l) |= *(cp)++; \
}
#define PUTLONG(l, cp) { \
	*(cp)++ = (u_char) ((l) >> 24); \
	*(cp)++ = (u_char) ((l) >> 16); \
	*(cp)++ = (u_char) ((l) >> 8); \
	*(cp)++ = (u_char) (l); \
}

#define INCPTR(n, cp)	((cp) += (n))
#define DECPTR(n, cp)	((cp) -= (n))

/*
 * System dependent definitions for user-level 4.3BSD UNIX implementation.
 */

#define TIMEOUT(r, f, t)	ppptimeout((r), (f), (t))
#define UNTIMEOUT(r, f)	pppuntimeout((r), (f))

#define BCOPY(s, d, l)		memcpy(d, s, l)
#define BZERO(s, n)		memset(s, 0, n)

#define PRINTMSG(m, l)		{ info("Remote message: %0.*v", l, m); }

/*
 * MAKEHEADER - Add Header fields to a packet.
 */
#define MAKEHEADER(p, t) { \
    PUTCHAR(PPP_ALLSTATIONS, p); \
    PUTCHAR(PPP_UI, p); \
    PUTSHORT(t, p); }

/*
 * Exit status values.
 */
#define EXIT_OK			0
#define EXIT_FATAL_ERROR	1
#define EXIT_OPTION_ERROR	2
#define EXIT_NOT_ROOT		3
#define EXIT_NO_KERNEL_SUPPORT	4
#define EXIT_USER_REQUEST	5
#define EXIT_LOCK_FAILED	6
#define EXIT_OPEN_FAILED	7
#define EXIT_CONNECT_FAILED	8
#define EXIT_PTYCMD_FAILED	9
#define EXIT_NEGOTIATION_FAILED	10
#define EXIT_PEER_AUTH_FAILED	11
#define EXIT_IDLE_TIMEOUT	12
#define EXIT_CONNECT_TIME	13
#define EXIT_CALLBACK		14
#define EXIT_PEER_DEAD		15
#define EXIT_HANGUP		16
#define EXIT_LOOPBACK		17
#define EXIT_INIT_FAILED	18
#define EXIT_AUTH_TOPEER_FAILED	19

/*
 * Debug macros.  Slightly useful for finding bugs in pppd, not particularly
 * useful for finding out why your connection isn't being established.
 */

#ifdef DEBUGALL
#define DEBUGMAIN	1
#define DEBUGFSM	1
#define DEBUGLCP	1
#define DEBUGIPCP	1
#define DEBUGIPV6CP	1
#define DEBUGUPAP	1
#define DEBUGCHAP	1
#endif
#define DEBUGMAIN	1
#define DEBUGUPAP	1
#define DEBUGCHAP	1


#ifdef DEBUGMAIN
#define MAINDEBUG(x)	if (debug) dbglog x
#else
#define MAINDEBUG(x)
#endif

#ifdef DEBUGSYS
#define SYSDEBUG(x)	if (debug) dbglog x
#else
#define SYSDEBUG(x)
#endif

#ifdef DEBUGFSM
#define FSMDEBUG(x)	if (debug) dbglog x
#else
#define FSMDEBUG(x)
#endif

#ifdef DEBUGLCP
#define LCPDEBUG(x)	if (debug) dbglog x
#else
#define LCPDEBUG(x)
#endif

#ifdef DEBUGIPCP
#define IPCPDEBUG(x)	if (debug) dbglog x
#else
#define IPCPDEBUG(x)
#endif

#ifdef DEBUGIPV6CP
#define IPV6CPDEBUG(x)  if (debug) dbglog x
#else
#define IPV6CPDEBUG(x)
#endif

#ifdef DEBUGUPAP
#define UPAPDEBUG(x)	if (debug) dbglog x
#else
#define UPAPDEBUG(x)
#endif

#ifdef DEBUGCHAP
#define CHAPDEBUG(x)	if (debug) dbglog x
#else
#define CHAPDEBUG(x)
#endif

#ifdef DEBUGIPXCP
#define IPXCPDEBUG(x)	if (debug) dbglog x
#else
#define IPXCPDEBUG(x)
#endif

#ifndef SIGTYPE
#if defined(sun) || defined(SYSV) || defined(POSIX_SOURCE)
#define SIGTYPE void
#else
#define SIGTYPE int
#endif /* defined(sun) || defined(SYSV) || defined(POSIX_SOURCE) */
#endif /* SIGTYPE */

#ifndef MIN
#define MIN(a, b)	((a) < (b)? (a): (b))
#endif
#ifndef MAX
#define MAX(a, b)	((a) > (b)? (a): (b))
#endif

#endif /* __PPP_H__ */
